.globl map
map:
    beq $a0, $0, done			# if a NULL pointer is given as a parameter then we are done.

    addi $sp, $sp, -12
    sw $ra, 0($sp)
    sw $s1, 4($sp)
    sw $s0, 8($sp)

    add $s0, $a0, $0			# put in $s0 the address of the current node 
    add $s1, $a1, $0			# put in $s1 the address of the function to apply
    add $t0, $0,  $0			# $t0 is a counter

    # remember that each node is 12 bytes long:
    #    4 bytes for the dynamic array's pointer, followed by
    #    4 bytes for the number of elements in the array, and
    #    4 more bytes for the pointer to the "next" field         

    # also keep in mind that we should make NO assumptions about which registers are 
    # modified by called functions, even when we know the contents of those functions. 
    # MIPS calling rules and conventions should ALWAYS apply.

mapLoop:
    add $t1, $s0, $0		    # put in $t1 the address of the array from the current node
    lw  $t2, 4($s0)			    # put in $t2 the size of the array 

    add $t1, $t1, $t0		    # adjust the array address by the counter value to get the address of the desired element in the array 
    lw $a0, 0($t1)			    # put in $a0 the value of the array element.
   
    jalr $s1				    # call the supplied function on this value.

    sw $v0, 0($t1)			    # store the returned value back to the array 
    addi $t0, $t0, 1			# increment the counter
    bne $t0, $t2, mapLoop		# repeat (i.e. loop) if array size not yet reached

    la $a0, 8($s0)			    # Load the address of the next node into $a0
    lw $a1, 0($s1)			    # Put back in $a1 the address of the mystery function to prepare for the recursive call

    jal  map				    # make a recursive call to map

    lw $s0, 8($sp)
    lw $s1, 4($sp)
    lw $ra, 0($sp)
    addi $sp, $sp, 12
done:
    jr $ra

.globl create_default_list
create_default_list:
    addi $sp, $sp, -4		
    sw $ra, 0($sp)
    
    move $s0, $a0		    	# $s0 = "arrays" address
    move $s1, $a1		    	# $s1 = number of elements in the array in a node
    move $s2, $a2           	# $s2 = number of nodes (=nbr of lines in "arrays")

    li $s3, 0				    # node pointer (initially = NULL)
    li $s4, 0			    	# number of processed nodes (initially = 0) 
        
loop:                                   # do{ ...
    li $a0, 12
    jal malloc			    	# allocate memory for the node.
    move $s5, $v0       		# $s5 = node address returned by malloc

    sll  $a0, $s1, 2
    jal  malloc			    	# allocate memory for the dynamic array
    sw $v0, 0($s5)		    	# node->arr = address returned by malloc

    lw $a0, 0($s5)
    move $a1, $s0
    move $a2, $s1
    jal fillArray		    	# fill-in the array "node->arr" with a series of ints 

    sw $s1, 4($s5)		    	# node->size = size of the array
    sw $s3, 8($s5)		    	# node->next = address of next node

    add  $s3, $0, $s5			# last = node

    add  $s0, $s0, $t0			# $s0 points to the next set of integers in "arrays"

    addi $s4, $s4, 1			# i++

    bne $s4, $s2, loop			# ... } while (i != number of nodes);

    move $v0, $s3                       # return in $v0 the "head" pointer of the linked list
    
    addi $sp, $sp,  4
    jr $ra

fillArray: 
    sll  $a2, $a2, 2			# $a2 = size (in bytes) of the array in the current node
next:    
    beq  $a2, $0, filled
    lw   $t0, 0($a1)			# put in $t0 a value from "arrays"
    sw   $t0, 0($a0)			# the value is stored in the dynamic array "node->arr"
    
    addi $a0, $a0, 4
    addi $a1, $a1, 4
    addi $a2, $a2,-4
    j next
    
filled:    
    jr $ra

malloc:				            # Make sure that when calling malloc, the amount of memory to allocate is in $a0
    li $v0, 9
    syscall
    jr $ra

